"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.processCommandPath = void 0;
const stacks_commands_1 = require("@takomo/stacks-commands");
const util_1 = require("@takomo/util");
const lodash_merge_1 = __importDefault(require("lodash.merge"));
const getDeploymentRole = (currentIdentity, configSetType, group, target) => {
    switch (configSetType) {
        case "bootstrap":
            const bootstrapRole = target.bootstrapRole ?? group.bootstrapRole;
            if (bootstrapRole) {
                return bootstrapRole;
            }
            const bootstrapRoleName = target.bootstrapRoleName ?? group.bootstrapRoleName;
            if (!bootstrapRoleName) {
                return undefined;
            }
            const bootstrapAccountId = target.accountId ?? currentIdentity.accountId;
            return {
                iamRoleArn: `arn:aws:iam::${bootstrapAccountId}:role/${bootstrapRoleName}`,
            };
        case "standard":
            const deploymentRole = target.deploymentRole ?? group.deploymentRole;
            if (deploymentRole) {
                return deploymentRole;
            }
            const deploymentRoleName = target.deploymentRoleName ?? group.deploymentRoleName;
            if (!deploymentRoleName) {
                return undefined;
            }
            const deploymentAccountId = target.accountId ?? currentIdentity.accountId;
            return {
                iamRoleArn: `arn:aws:iam::${deploymentAccountId}:role/${deploymentRoleName}`,
            };
        default:
            throw new Error(`Unsupported config set type: ${configSetType}`);
    }
};
const deploy = async (credentialManager, commandPath, groupPath, targetName, input, io, configRepository, ctx) => {
    const result = await stacks_commands_1.deployStacksCommand({
        input,
        credentialManager,
        configRepository,
        ctx,
        io: io.createStackDeployIO(`${groupPath}/${targetName}`),
    });
    return {
        commandPath,
        result,
        status: result.status,
        success: result.success,
        message: result.message,
    };
};
const undeploy = async (credentialManager, commandPath, groupPath, targetName, input, io, configRepository, ctx) => {
    const result = await stacks_commands_1.undeployStacksCommand({
        input,
        credentialManager,
        configRepository,
        ctx,
        io: io.createStackUndeployIO(`${groupPath}/${targetName}`),
    });
    return {
        commandPath,
        result,
        status: result.status,
        success: result.success,
        message: result.message,
    };
};
const deployOrUndeploy = async (variables, credentialManager, operation, commandPath, groupPath, target, timer, io, ctx, stacksConfigRepository) => {
    const input = {
        timer,
        commandPath,
        ignoreDependencies: false,
        interactive: false,
    };
    const commandContext = {
        ...ctx.commandContext,
        variables,
    };
    switch (operation) {
        case "deploy":
            return deploy(credentialManager, commandPath, groupPath, target, input, io, stacksConfigRepository, commandContext);
        case "undeploy":
            return undeploy(credentialManager, commandPath, groupPath, target, input, io, stacksConfigRepository, commandContext);
        default:
            throw new Error(`Unknown operation: ${operation}`);
    }
};
const processCommandPath = async (holder, group, target, configSet, commandPath, timer, state, stacksConfigRepository) => {
    const { io, ctx, input: { operation, configSetType }, } = holder;
    const { variables } = ctx;
    io.info(`Execute command path: ${commandPath}`);
    if (state.failed) {
        timer.stop();
        return {
            commandPath,
            result: {
                message: "Cancelled",
                status: "CANCELLED",
                timer,
                success: false,
                results: [],
            },
            status: "CANCELLED",
            success: false,
            message: "Cancelled",
        };
    }
    const mergedVars = util_1.deepCopy(variables.var);
    lodash_merge_1.default(mergedVars, target.vars);
    const commandPathVariables = {
        env: variables.env,
        var: mergedVars,
        context: variables.context,
        target: {
            name: target.name,
            accountId: target.accountId,
        },
    };
    const currentIdentity = await ctx.credentialManager.getCallerIdentity();
    const deploymentRole = getDeploymentRole(currentIdentity, configSetType, group, target);
    const credentialManager = deploymentRole
        ? await ctx.credentialManager.createCredentialManagerForRole(deploymentRole.iamRoleArn)
        : ctx.credentialManager;
    if (target.accountId) {
        const identity = await credentialManager.getCallerIdentity();
        if (identity.accountId !== target.accountId) {
            throw new util_1.TakomoError(`Current credentials belong to AWS account ${identity.accountId}, but the deployment target can be deployed only to account: ${target.accountId}`);
        }
    }
    try {
        return deployOrUndeploy(commandPathVariables, credentialManager, operation, commandPath, group.path, target.name, timer, io, ctx, stacksConfigRepository);
    }
    catch (e) {
        io.error(`Unhandled error when executing group: ${group.path}, target: ${target.name}, config set: ${configSet.name}, command path: ${commandPath}`, e);
        timer.stop();
        return {
            commandPath,
            result: {
                message: e.message,
                status: "FAILED",
                timer,
                success: false,
                results: [],
            },
            success: false,
            status: "FAILED",
            message: "Failed",
        };
    }
};
exports.processCommandPath = processCommandPath;
//# sourceMappingURL=command-path.js.map